/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/

#include "tlv320aic23b.h"
#include "startup_sam7.h"
#include "i2s.h"

#include "spi.h"
#include "config_spi.h"
#define tlv320aic23b_dev    SPI_0

#include "AT91SAM7S64.h"
#define __inline extern inline
#include "lib_AT91SAM7S64.h"




/* left and right Line In control */
#define REG_LLINEIN     0x00
#define REG_RLINEIN     0x01
#define SIMULTANE       (1<<8)
#define LINEIN_MUTE     (1<<7)
#define LINEIN_VOLUME   0x01F
/* left and right headphone output control */
#define REG_LHP         0x02
#define REG_RHP         0x03
#define HP_CROSSDETECT  (1<<7)
#define HP_VOLUME       0x07F
/* analog audio path control */
#define REG_ANALOG      0x04
#define SIDETONE_EN     (1<<5)
#define SIDETONE_ATT    (0x1C0)
#define DAC_EN          (1<<4)
#define BYP_EN          (1<<3)
#define INPUT_MIC       (1<<2)
#define INPUT_LINE      (0<<2)
#define MIC_MUTE        (1<<1)
#define MIC_BOOST       (1<<0)
/* digital audio path control */
#define REG_DIGITAL     0x05
#define DAC_MUTE        (1<<3)
#define DEEMP_DIS       (0<<1)
#define DEEMP_32kHz     (1<<1)
#define DEEMP_44kHz     (2<<1)
#define DEEMP_48kHz     (3<<1)
#define ADC_HP_DIS      (1<<0)
/* power control, the values must be anded! */
#define REG_POWER       0x06
#define PWR_OFF         (0x1FF)
#define PWR_ON          (~(1<<7))
#define PWR_CLK         (~(1<<6))
#define PWR_OSC         (~(1<<5))
#define PWR_OUT         (~(1<<4))
#define PWR_DAC         (~(1<<3))
#define PWR_ADC         (~(1<<2))
#define PWR_MIC         (~(1<<1))
#define PWR_LINE        (~(1<<0))
/* digital audio interface format */
#define REG_INTERFACE   0x07
#define MSTR            (1<<6)
#define LR_SWAP         (1<<5)
#define LR_PHASE        (1<<4)
#define DATLEN_16       (0<<2)
#define DATLEN_20       (1<<2)
#define DATLEN_24       (2<<2)
#define DATLEN_32       (3<<2)
#define DATFORM_DSP     (3<<0)
#define DATFORM_I2S     (2<<0)
#define DATFORM_MSBL    (1<<0)
#define DATFORM_MSBR    (0<<0)
/* sample rate control */
#define REG_SAMPLERATE  0x08
#define CLK_INDIV       (1<<7)
#define CLK_OUTDIV      (1<<6)
#define SAMPLERATE_USB_44   ((1<<0) | (0x11<<1))
#define SAMPLERATE_USB_48   ((1<<0) | (0x00<<1))
#define SAMPLERATE_USB_8    ((1<<0) | (0x06<<1))
#define SAMPLERATE_USB_96   ((1<<0) | (0x0E<<1))
#define SAMPLERATE_USB_88   ((1<<0) | (0x1F<<1))
#define SAMPLERATE_USB_32   ((1<<0) | (0x0C<<1))
/* digital interface activation register */
#define REG_ACTIVATE    0x09
#define ACTIVATE        (1<<0)
/* reset register */
#define REG_RESET       0x0F
#define RESET           0x000



void write_reg( unsigned char reg, unsigned int data ) {
    /* this function pretends, the device is already selected.
    * select the device always in top-level-functions! */
    /* select the device */
    unsigned int sync = SPI_SYNCHRONIZE();
    spi_sync_start( tlv320aic23b_dev );
    spi_sync_tmit( (reg << 9) | (data & 0x1FF) );
    spi_sync_end();
    SPI_DESYNCHRONIZE( sync );
}


void tlv320aic23b_init( void ) {
    /* select the device */
//    spi_start( tlv320aic23b_dev );
    /* reset the device */
    write_reg( REG_RESET, RESET );
    /* power up the devices we need */
    write_reg( REG_POWER, PWR_MIC & PWR_OUT & PWR_DAC & PWR_ADC & PWR_ON);
    /* initialize I2S on our side */
    i2s_init();
    /* initialize clock (MCK) */
    AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA, 0, AT91C_PA23_PWM0 );
    AT91F_PWMC_CfgPMC();
    AT91C_BASE_PWMC->PWMC_CH[0].PWMC_CMR = 0;   /* MCK */
    AT91C_BASE_PWMC->PWMC_CH[0].PWMC_CPRDR = 4; /* period is 2 */
    AT91C_BASE_PWMC->PWMC_CH[0].PWMC_CDTYR = 2; /* duty cycle 1/2 */
    AT91C_BASE_PWMC->PWMC_ENA = AT91C_PWMC_CHID0;   /* enable */
    /* switch to ADC, DAC, Mic Boost, Input=Mic and 48kHz */
    write_reg( REG_ANALOG, DAC_EN | MIC_BOOST | INPUT_MIC );
    write_reg( REG_DIGITAL, DEEMP_48kHz );
    write_reg( REG_SAMPLERATE, SAMPLERATE_USB_48 );
    /* initialize I2S on codec side */
    write_reg( REG_INTERFACE, MSTR | DATLEN_16 | DATFORM_I2S );
    /* activate the interface */
    write_reg( REG_ACTIVATE, ACTIVATE );
//    spi_end();
}

// left and right are in DB (+6 .. -73, 3dB steps)
void tlv320aic23b_vol_hp( int left, int right ) {
    /* set the left and right volume controls to the specified values */
//    spi_start( tlv320aic23b_dev );
    if (left > 6) left = 6;
    if (left < -73) left = -73;
    if (right > 6) left = 6;
    if (right < -73) left = -73;
    left = (left / 3) + 0x7D;
    right = (right / 3) + 0x7D;
    if (left == right) {
        write_reg( REG_LHP, SIMULTANE | (left & HP_VOLUME) );
    } else {
        write_reg( REG_LHP, (left & HP_VOLUME) );
        write_reg( REG_RHP, (right & HP_VOLUME) );
    }
//    spi_end();
}

void tlv320aic23b_powerdown( void ) {
    /* shutdown the device and I2S interface */
//    spi_start( tlv320aic23b_dev );
    write_reg( REG_POWER, PWR_OFF );
//    spi_end();
    /* disable CLOCK */
    AT91C_BASE_PWMC->PWMC_DIS = AT91C_PWMC_CHID0;
    /* disable I2S */
    i2s_powerdown();
}
