/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/
/*
 * Startup Code for ARM
 * ARM-Defines for Assembler
 *
 */

#ifndef _ARM_INC_
#define _ARM_INC_

//
// ******************************************************************************
//                ARM related constants
// ******************************************************************************
#define ARM_MODE_User		(0x10)
#define ARM_MODE_FIQ		(0x11)
#define ARM_MODE_IRQ		(0x12)
#define ARM_MODE_Supervisor	(0x13)
#define ARM_MODE_Abort		(0x17)
#define ARM_MODE_Undefined	(0x1B)
#define ARM_MODE_System		(0x1F)

#define ARM_I_BIT		(0x80)
#define ARM_F_BIT		(0x40)
#define ARM_T_BIT		(0x20)

#if defined(ASSEMBLER)
/* the following defines are always assembled in ARM-Mode.
 * the function can be called either in ARM, THUMB or THUMB-Interwork
 */
#if defined(ARM_MODE_THUMB)
	/* all C-Functions are compiled without Thumb-Interwork.
	 * So make shure that Interwork is supported by the Caller!
	 */
	.macro CALL_FUNC	func
		ldr	r12, =\func
		mov	lr, pc
		b	CALLSTUB_THUMB
	.endm
	.macro CALL_REG		reg
		mov 	r12, \reg
		mov	lr, pc
		b	CALLSTUB_THUMB
	.endm
#elif defined(ARM_MODE_THUMB_INTERWORK)
	/* all C-Functions are compiled with Thumb-Interwork.
	 * calling can be done by bx, return is also always bx */
	.macro CALL_FUNC	func
		ldr	r12, =\func
		mov	lr, pc
		bx	r12
	.endm
	.macro CALL_REG		reg
		mov	lr, pc
		bx	\reg
	.endm
#else
	/* C-Functions are compiled in ARM-Mode */
	.macro CALL_FUNC	func
		bl	\func
	.endm
	.macro CALL_REG		reg
		/* indirect call only via BX possible on armv4T */
		mov	lr, pc
		bx	\reg
	.endm
#endif
#endif /* defined(ASSEMBLER) */

#endif
