/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/
/** @file config_spi.h
 * @brief SPI-Library
 * @author Clemens Helfmeier
 * @date 25.06.2006
 * 
 * This file is architecture dependend.
 * It defines the selectable chips on architecture level
 * change the options below for correct use.
 *
 * THIS FILE IS FOR USE WITH THE AT91SAM7 SERIES.
 * It must be totally rewritten for different controller.
 */


#ifndef _CONFIG_SPI_H_
#define _CONFIG_SPI_H_

#ifdef __cplusplus
extern "C" {
#endif

/** Define which chip select lines should be used on function A. */
#define SPI_CSLINES_A	(0)
/** Define which chip select lines should be used on function B. */
#define SPI_CSLINES_B	(0)
/** Define which chip select lines should be used on general io-pin function. */
#define SPI_CSLINES_GPIO (SPI_CS_0 | SPI_CS_1 | SPI_CS_2 | SPI_CS_3 )	

/* the way, chip select registers are written */
/* tlv320aic23b */
/** Chip select register for device 0. */
#define SPI_CSR_0		(AT91C_SPI_CPOL | AT91C_SPI_BITS_16 | ((255<<8) & AT91C_SPI_SCBR))
/** The spi device 0. This must be passed to spi_start */
#define SPI_0			(0)
/** The spi device 0 has hardware cs line handling */
#define SPI_CS_0		(AT91C_PA11_NPCS0)

/* enc28j60 */
/** Chip select register for device 1. */
#define SPI_CSR_1		(AT91C_SPI_NCPHA | AT91C_SPI_BITS_8 | ((5<<8) & AT91C_SPI_SCBR) | ((0<<16) & AT91C_SPI_DLYBS) | ((0<<24) & AT91C_SPI_DLYBCT))
/** The spi device 1. This must be passed to spi_start */
#define SPI_1			(1)
/** The spi device 1 has software cs line handling */
#define SPI_CS_1		(AT91C_PA9_NPCS1)

/* sd-card (not yet tested)*/
/** Chip select register for device 2. */
#define SPI_CSR_2		(0)
/** The spi device 2. This must be passed to spi_start */
#define SPI_2			(2)
/** The spi device 2 has hardware cs line handling */
#define SPI_CS_2		0
    // AT91C_PA10_NPCS2


/* empty */
/** Chip select register for device 3. */
#define SPI_CSR_3		(0)
/** The spi device 3. This must be passed to spi_start */
#define SPI_3			(3)
/** The spi device 3 has hardware cs line handling */
#define SPI_CS_3		0
    // AT91C_PA10_NPCS2

#ifdef __cplusplus
}
#endif
#endif
