/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/
/*
 * Startup Code for ARM
 *
 * Assembler file for FIQ.
 * this _must_ be adapted if the FIQ should be used
 * effectivley. Thus the current handler does _nothing_.
 *
 * by: Clemens Helfmeier
 * ClemensHelfmeier {at] gmx `dot" de
 * (replace the corresponding at and dot in the email, please).
 */

#define ASSEMBLER
#include "arm.inc"

.text
	.func FIQ_Handler
	.global FIQ_Handler
FIQ_Handler:
	/* handle the Fast Interrupt Request
	 * Registers 0..7 are used by the Application
	 * Registers 8..12 are usable,
	 * Register 13 is the FIQ-Stack pointer
	 * Register 14 contains the return address
	 *
	 * TODO:
	 *   Acknowledge the Interrupt
	 *   save registers that will be used (0..7,14)
	 *   do whatever the interrupt handler should do
	 */
    
    /* this handler does not support nested interrupts.
     * There is only one handler function.
     * The routine can use any Register (R0-R7 are saved on the FIQ stack).
     * The stack is still the fiq stack when the handler is called.
     */
    
    .extern fiq_handler
    
    /* save register */
    stmfd   sp!, {r0-r7}
    /* call service routine */
/*    CALL_FUNC fast_interrupt_handler */
    /* restore registers that where saved on the stack */
    ldmia   sp!, {r0-r7}

	/* return from FIQ-Handler */
	subs	pc, lr, #4

.size FIQ_Handler, . - FIQ_Handler
.endfunc
