/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/

#include "i2s.h"

#include "system/error.h"

#include "debug/debug.h"
#define DBG_LEVEL   DBG_LEVEL_ERROR
#include "debug/debug.h"

#include "AT91SAM7S64.h"
#define __inline extern inline
#include "lib_AT91SAM7S64.h"

/* private prototypes */
void tlv320aic23b_interrupt( void );

/* private variables */
void (*txHandler)( void ) = 0;
void (*rxHandler)( void ) = 0;

void i2s_init( void ) {
	/* initialize the I2S-Interface of the SAM7S */
	AT91F_SSC_CfgPMC();

	/* disable reciever and transmitter before configuration */
	AT91C_BASE_SSC->SSC_CR = AT91C_SSC_SWRST | AT91C_SSC_TXDIS | AT91C_SSC_RXDIS;

	/* set io-lines to be used by peripheral */
	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA, AT91C_PA17_TD | AT91C_PA15_TF | AT91C_PA16_TK | AT91C_PA18_RD | AT91C_PA20_RF, 0 );

	/* select clock options */
	AT91C_BASE_SSC->SSC_CMR = 0x00;	/* we don't need the divider */
	AT91C_BASE_SSC->SSC_RCMR = AT91C_SSC_CKS_TK | AT91C_SSC_START_EDGE_RF | ((1<<16) & AT91C_SSC_STTDLY) | AT91C_SSC_CKI;	/* reciever clock is the same as transmitter clock */
	AT91C_BASE_SSC->SSC_RFMR = (15 & AT91C_SSC_DATLEN) | (AT91C_SSC_MSBF) | ((0<<8) & AT91C_SSC_DATNB);
	AT91C_BASE_SSC->SSC_TCMR = AT91C_SSC_CKS_RK | AT91C_SSC_START_EDGE_RF | ((0<<16) & AT91C_SSC_STTDLY);	/* this is Recieve (the) clocK */
	AT91C_BASE_SSC->SSC_TFMR = (15 & AT91C_SSC_DATLEN) | AT91C_SSC_MSBF | ((0<<8) & AT91C_SSC_DATNB); // | ((1<<8) & AT91C_SSC_DATNB) | ((16<<16) & AT91C_SSC_FSLEN) | AT91C_SSC_FSOS_TOGGLE | AT91C_SSC_FSDEN;

	/* configure the interrupt, disable all by default */
    AT91C_BASE_SSC->SSC_IDR = -1; 
    AT91F_AIC_CfgPMC();
	AT91F_AIC_ConfigureIt( AT91C_BASE_AIC, AT91C_ID_SSC, 4 /*priority*/, AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED, tlv320aic23b_interrupt );
    AT91F_AIC_DisableIt( AT91C_BASE_AIC, AT91C_ID_SSC );

    // open the PDC
    AT91F_PDC_Open( AT91C_BASE_PDC_SSC );
    
    /* enable the SSC */
    AT91C_BASE_SSC->SSC_CR = AT91C_SSC_TXEN | AT91C_SSC_RXEN;
}

void i2s_powerdown( void ) {
    /* close the pdc */
    AT91F_PDC_Close( AT91C_BASE_PDC_SSC );
    /* switch off the I2S interface */
	AT91C_BASE_SSC->SSC_CR = AT91C_SSC_TXDIS | AT91C_SSC_RXDIS;
	/* turn off interrupt */
	AT91F_AIC_DisableIt( AT91C_BASE_AIC, AT91C_ID_SSC );
	/* disable clock */
	AT91F_PMC_DisablePeriphClock( AT91C_BASE_PMC, AT91C_ID_SSC );
}

int i2s_r( void ) {
	/* wait until data arrive */
	while (!(AT91C_BASE_SSC->SSC_SR & AT91C_SSC_RXRDY) );
	/* return the data */
	return AT91C_BASE_SSC->SSC_RHR;
}

void i2s_t( int data ) {
	/* wait until transmit register empty */
	while (!(AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXEMPTY) );
	/* send data */
	AT91C_BASE_SSC->SSC_THR = data;
}

void i2s_setTxEmptyHandler( void (*handler)( void ) ) {
	if (handler == 0) {
		/* disable the interrupt */
		AT91C_BASE_SSC->SSC_IDR = AT91C_SSC_ENDTX;
        txHandler = 0;
	} else {
		/* the AIC is programmed on init, just keep the handler and activate the interrupt */
		txHandler = handler;
		AT91C_BASE_SSC->SSC_IER = AT91C_SSC_ENDTX;
	}
}

void i2s_setRxFullHandler( void (*handler)( void ) ) {
	if (handler == 0) {
		/* disable the interrupt */
		AT91C_BASE_SSC->SSC_IDR = AT91C_SSC_ENDRX;
        rxHandler = 0;
	} else {
		/* the AIC is programmed on init, just keep the handler and activate the interrupt */
		rxHandler = handler;
        DBG_V_S( "i2s_setRxFullHandler: enabling interrupt.\n" );
        AT91C_BASE_SSC->SSC_IER = AT91C_SSC_ENDRX;
	}
}

void tlv320aic23b_interrupt( void ) {
	/* this is the interrupt handler for SSC */
    int sr = AT91C_BASE_SSC->SSC_SR;
    if (((sr & AT91C_SSC_ENDRX) != 0) && (rxHandler != 0)) {
        rxHandler();
    } else if (((sr & AT91C_SSC_ENDTX) != 0) && (txHandler != 0)) {
        txHandler();
    }
}

/** add the given buffer to the list of buffers to be written to
 * ( DMA )
 */
ERROR i2s_addRxBuffer( void * buf, unsigned int len ) {
#if 0
    if (AT91F_PDC_ReceiveFrame( AT91C_BASE_PDC_SSC, 0, 0, buf, len ) == 0) {
        return ERROR_FAILURE;
    } else {
        return ERROR_SUCCESS;
    } 
#else
    DBG_V_S( "i2s_addRxBuffer( " ); DBG_V_I( buf ); DBG_V_S( ", " ); DBG_V_I( len ); DBG_V_S( " ).\n" );
    ERROR ret = ERROR_SUCCESS;
    AT91F_PDC_DisableRx( AT91C_BASE_PDC_SSC );
    if (AT91C_BASE_PDC_SSC->PDC_RCR == 0) {
        AT91C_BASE_PDC_SSC->PDC_RPR = (int)buf;
        AT91C_BASE_PDC_SSC->PDC_RCR = len;
    } else if (AT91C_BASE_PDC_SSC->PDC_RNCR == 0) {
        AT91C_BASE_PDC_SSC->PDC_RNPR = (int)buf;
        AT91C_BASE_PDC_SSC->PDC_RNCR = len;
    } else {
        ret = ERROR_FAILURE;
    }
    AT91F_PDC_EnableRx( AT91C_BASE_PDC_SSC );
    return ret;
#endif
}


/** add the given buffer to the list of buffers to be read from
 * ( DMA )
 */
ERROR i2s_addTxBuffer( void * buf, unsigned int len ) {
#if 0
    if (AT91F_PDC_SendFrame( AT91C_BASE_PDC_SSC, 0, 0, buf, len ) == 0) {
        return ERROR_FAILURE;
} else {
        return ERROR_SUCCESS;
} 
#else
    DBG_V_S( "i2s_addTxBuffer( " ); DBG_V_I( buf ); DBG_V_S( ", " ); DBG_V_I( len ); DBG_V_S( " ).\n" );
    ERROR ret = ERROR_SUCCESS;
    AT91F_PDC_DisableTx( AT91C_BASE_PDC_SSC );
    if (AT91C_BASE_PDC_SSC->PDC_TCR == 0) {
        AT91C_BASE_PDC_SSC->PDC_TPR = (int)buf;
        AT91C_BASE_PDC_SSC->PDC_TCR = len;
    } else if (AT91C_BASE_PDC_SSC->PDC_TNCR == 0) {
        AT91C_BASE_PDC_SSC->PDC_TNPR = (int)buf;
        AT91C_BASE_PDC_SSC->PDC_TNCR = len;
    } else {
        ret = ERROR_FAILURE;
    }
    AT91F_PDC_EnableTx( AT91C_BASE_PDC_SSC );
    return ret;
#endif
}

void i2s_disableInterrupt( void ) {
    AT91F_AIC_DisableIt( AT91C_BASE_AIC, AT91C_ID_SSC );
}
void i2s_enableInterrupt( void ) {
    AT91F_AIC_EnableIt( AT91C_BASE_AIC, AT91C_ID_SSC );
}

/** return the number of buffers still in the txer
 */
unsigned int i2s_TxEmptyBufferCount( void ) {
    unsigned int ret = 2;
    AT91F_PDC_DisableTx( AT91C_BASE_PDC_SSC );
    if (AT91C_BASE_PDC_SSC->PDC_TNCR != 0) {
        ret = 0;
    } else if (AT91C_BASE_PDC_SSC->PDC_TCR != 0) {
        ret = 1;
    }
    AT91F_PDC_EnableTx( AT91C_BASE_PDC_SSC );
    return ret;
}

/** return the number of buffers still in the rxer
 */
unsigned int i2s_RxEmptyBufferCount( void ) {
    unsigned int ret = 2;
    AT91F_PDC_DisableRx( AT91C_BASE_PDC_SSC );
    if (AT91C_BASE_PDC_SSC->PDC_RNCR != 0) {
        ret = 0;
    } else if (AT91C_BASE_PDC_SSC->PDC_RCR != 0) {
        ret = 1;
    }
    AT91F_PDC_EnableRx( AT91C_BASE_PDC_SSC );
    return ret;
}
