/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/


#ifndef _INTERRUPT_H_
#define _INTERRUPT_H_


#include "controller/arm.inc"


#define DISABLE_ALL_INTERRUPTS()        int_disable()
#define ENABLE_ALL_INTERRUPTS( enable ) int_restore( enable )

static inline unsigned int int_disable( void ) {
    /* turn off irq and fiq and return a value that represents the state before the turn off. */
    unsigned int ret;
    __asm volatile(
        "mrs %0, cpsr\n\t"
        "orr r1, %0, %1\n\t"   /* disable interrupts*/
        "msr cpsr_c, r1\n\t"
        : /*outputs*/ "=r" (ret)
        : /*inputs*/ "I" (ARM_F_BIT | ARM_I_BIT)
        : /*clobbers*/ "r1"
    );
    return ret;
}

static inline void int_restore( unsigned int enable ) {
    enable |= ~(ARM_F_BIT | ARM_I_BIT);
    __asm volatile(
        "mrs r1, cpsr\n\t"
        "orr r1, r1, %1\n\t"    /* first disable alle */
        "and r1, r1, %0\n\t"    /* then enable the ones given interrupts*/
        "msr cpsr_c, r1\n\t"
        : /*outputs*/
        : /*inputs*/ "r" (enable), "I" (ARM_F_BIT | ARM_I_BIT)
        : /*clobbers*/ "r1"
    );
}







#endif
