/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/
/** This file contains functions used with the SPI interface of the AT91SAM7-Series 
 * of microcontrollers. Most of the functions are inline functions, see spi.h for details.
 */

#include "controller/spi.h"
#include "controller/config_spi.h"


const unsigned int cs_modes[ 4 ] = { SPI_CS_0, SPI_CS_1, SPI_CS_2, SPI_CS_3 };

ERROR spi_init( void ) {
    /* start the clock for SPI */
    AT91F_SPI_CfgPMC();
    /* Reset SPI */
    AT91F_SPI_Reset( AT91C_BASE_SPI );
    /* initialize PIO to drive IO signals */
    AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA, AT91C_PA13_MOSI | AT91C_PA12_MISO | AT91C_PA14_SPCK, 0 );
    /* initialize PIO to drive chip select signals */
    AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA, SPI_CSLINES_A, SPI_CSLINES_B );
    /* for software chip select, put all the lines up */
    AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, SPI_CS_0 | SPI_CS_1 | SPI_CS_2 | SPI_CS_3 );
    AT91F_PIO_SetOutput( AT91C_BASE_PIOA, SPI_CS_0 | SPI_CS_1 | SPI_CS_2 | SPI_CS_3 );
    /* stop the PDC */
    AT91F_PDC_Open( AT91C_BASE_PDC_SPI );
    /* configure Mode Register */
    AT91C_BASE_SPI->SPI_MR =
            AT91C_SPI_MSTR |    /* we use MASTER only mode */
            AT91C_SPI_PS_FIXED |    /* we only use Fixed Peripheral Select */
            AT91C_SPI_MODFDIS |     /* no Fault protection since there is no second master */
            AT91C_SPI_PCS |     /* no peripheral selected */
            ((10<<24) & AT91C_SPI_DLYBCS);  /* 10 Clock Cycles betwenn two different chip selects */
    /* program all chip select registers correctly */
    AT91C_BASE_SPI->SPI_CSR[0] = SPI_CSR_0;
    AT91C_BASE_SPI->SPI_CSR[1] = SPI_CSR_1;
    AT91C_BASE_SPI->SPI_CSR[2] = SPI_CSR_2;
    AT91C_BASE_SPI->SPI_CSR[3] = SPI_CSR_3;

    /* enable the interrupt */
    AT91C_BASE_SPI->SPI_IDR = -1; 
    AT91F_AIC_CfgPMC();
    AT91F_AIC_ConfigureIt( AT91C_BASE_AIC, AT91C_ID_SPI, 3 /*priority*/, AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, spi_interrupt_handler );
    AT91F_AIC_EnableIt( AT91C_BASE_AIC, AT91C_ID_SPI );
    
    /* enable SPI */
    AT91F_SPI_Enable( AT91C_BASE_SPI );
    
    return ERROR_SUCCESS;
}


void spi_sync_start( unsigned int slave ) {
    AT91C_BASE_SPI->SPI_MR = (AT91C_BASE_SPI->SPI_MR & (~AT91C_SPI_PCS)) | ((~((1<<(slave)<<16))) & AT91C_SPI_PCS);
    // workaround for silicon bug. the data length is sometimes taken from SPI_CSR[0] instead of the correct
    if (slave != 0) {
        AT91C_BASE_SPI->SPI_CSR[0] = AT91C_BASE_SPI->SPI_CSR[ slave ];
    } else {
        AT91C_BASE_SPI->SPI_CSR[0] = SPI_CSR_0;
    }
    // pull low the cs-line. this is always done by the PIO
    AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, cs_modes[ slave ] );
    // empty the RDR
    volatile unsigned int i = AT91C_BASE_SPI->SPI_RDR;
    i = AT91C_BASE_SPI->SPI_SR;
}


