/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/
/*
 * Startup Code for ARM
 *
 * C-Code for Initialization
 * this code intializes the .bss and .data sections. It also call constructors
 *
 * by: Clemens Helfmeier
 * ClemensHelfmeier {at] gmx `dot" de
 * (replace the corresponding at and dot in the email, please).
 */

#include "startup_config.h"


extern unsigned int _data_start, _data_load_start, _data_end, _bss_start, _bss_end;
	/* the handling of the addresses is a bit complicated.
	 * gcc assumes the values by the linker are addresses (eg 0x200000) to the value they should
	 * represent.
	 * thus a simple unsigned int *pointer = _data_start; will result in loading the value
	 * at the address (0x200000). this is not what is intended to be done, we want 0x200000 in
	 * pointer - pointer should point to 0x200000.
	 * this is achived by replacing _data_start with &_data_start.
	 */

/* Init_BSS_DATA
 * load .data from ROM to RAM
 * zero out .bss
 * The routine pretends, all addresses are 4-Byte-aligned. Insert ". = ALIGN( 4 );" before
 * the definition of the PROVIDE( ... ); statement in the linker file to achieve this.
 * this procedure is executed in System Mode
 */
void  Init_BSS_DATA( void ) {
	unsigned int *dp, *sp;
	/* load the .data section into ram: */
	dp = &_data_start;
	sp = &_data_load_start;
	while (dp < &_data_end)
		*dp++ = *sp++;
	/* zero out the .bss section */
	dp = &_bss_start;
	while (dp < &_bss_end)
		*dp++ = 0;
}


extern unsigned int _ctors_start, _ctors_end;
	/* same here like with the values above;
	 */
/* Init_Constructors
 * initialize (call) all constructors of global objects
 */
void Init_Constructors( void ) {
	unsigned int *ctor = &_ctors_start;
	while (ctor < &_ctors_end) {
		((void (*)(void))*ctor)();
		ctor++;
	}
}

#if defined(DESTRUCTORS) && (DESTRUCTORS == 1)
extern unsigned int _dtors_start, _dtors_end;
	/* same here like with the values above;
	 */
/* Init_Destructors
 * initialize (call) all destructors of global objects
 */
void Init_Destructors( void ) {
	unsigned int *dtor = &_dtors_start;
	while (dtor < &_dtors_end) {
		((void (*)(void))*dtor)();
		dtor++;
	}
}
#endif
