/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/

/** @file debug.c
 * @author Clemens Helfmeier
 * @date 25.06.06
 * @brief This file contains functions that support debugging of the target program. This is usually achieved by sending data over some sort of serial line.
 *
 * This file must be modified when ported to a different hardware platform.
 */

#include "debug.h"
#include "usart.h"

void dbg_init( void ) {
    usart_init();
}

/** @brief Send a byte as number in the form 0x3A to the debugging terminal.
 * @private
 */
void dbg_putb( char x ) {
	usart_puts( "0x" );
	if ((x>>4) > 9) usart_putc( (x>>4) + 'A' - 10 );
	else usart_putc( (x>>4) + '0' );
	if ((x&0x0F) > 9) usart_putc( (x&0x0F) + 'A' - 10 );
	else usart_putc( (x&0x0F) + '0' );
}

/** @brief Send an integer as a number in the form 0x32a93a4b to the debugging terminal.
 * @private
 */
void dbg_puti( int i ) {
	usart_puts( "0x" );
	for (unsigned char x = 0; x<8; x++) {
		if (((i>>28)&0x0F) > 9) usart_putc( ((i>>28)&0x0F) + 'A' - 10 );
		else usart_putc( ((i>>28)&0x0F) + '0' );
		i = i << 4;
	}
}

/** @brief Send a string to the debugging termial. The string must be null-terminated.
 * @private
 */
void dbg_puts( char* s ) {
	usart_puts( s );
}
