#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>

#define BUFLEN 256
#define PORT 9936

void diep(char *s)
{
    perror(s);
    exit(1);
}

int main(void)
{
    struct sockaddr_in *sock_server, *sock_client;
    int s;
    socklen_t socklen = sizeof(struct sockaddr_in);

    sock_server = (struct sockaddr_in *) malloc( socklen );
    sock_client = (struct sockaddr_in *) malloc( socklen );

    if ( !sock_server || !sock_client )
        diep( "allocation failed" );

    if ((s = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
        diep("socket()");

    sock_server->sin_family = AF_INET;
    sock_server->sin_port = htons(PORT);
    sock_server->sin_addr.s_addr = htonl(INADDR_ANY);

    sock_client->sin_family = AF_INET;
    sock_client->sin_port = htons(PORT);
    if (inet_aton("192.168.6.20", &(sock_client->sin_addr)) == 0) {

        fprintf(stderr, "inet_aton() failed\n");
        exit(1);

    }
    
    if ( bind(s, (struct sockaddr *) sock_server, socklen ) == -1)
        diep("bind()");

    printf( "waiting for input.\n" );
      
    // initialize audio
    int audio_fd; 
    if ((audio_fd = open("/dev/dsp", O_RDONLY, 0)) == -1) 
    { /* Opening device failed */ 
        perror("/dev/dsp"); 
        exit(1); 
    }  
    
    int format = 0;
    int mask;
    int stereo = 1;     /* 0=mono, 1=stereo */
    int speed = 96000;

    // check for format
    if (ioctl(audio_fd, SNDCTL_DSP_GETFMTS, &mask) == -1) {
        perror("Couldn't get capabilities of soundcard.");
        exit(2);
    }
    if ((mask & AFMT_S16_LE) != 0) {
        format = AFMT_S16_LE;
        printf("using signed 16 bit little endian sound format.\n" );
    } else if ((mask & AFMT_S8) != 0) {
        format = AFMT_S8;
        printf("using signed 8 bit sound format.\n" );
    }
    if (format == 0) {
        printf("No usable sound format supported by soundcard, trying s16 be");
        format = AFMT_S16_BE;
    }
    // select format
    if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &format)==-1) { 
        perror("SNDCTL_DSP_SETFMT"); 
        exit(2); 
    } 
    if (format != AFMT_S16_LE) { 
        printf("The device doesn't support the requested audio format. The program should use another format (for example the one returned in 'format') or alternatively it must display an error message and to abort.\n");
        exit(3);
    }
    // select number of channels
    if (ioctl(audio_fd, SNDCTL_DSP_STEREO, &stereo)==-1) {
        perror("Could not select correct channel count.");
        exit(4);
    }
    if (stereo != 1) {
        printf( "Sound device doesn't support stereo mode.\n" );
    }
    // select speed
    int speed_ = speed;
    if (ioctl(audio_fd, SNDCTL_DSP_SPEED, &speed_)==-1) { 
        perror("Could not select speed"); 
        exit(5); 
    } 
    if (speed_ != speed) { 
        printf( "failed to set speed.\n" ); 
    }

    char buf[BUFLEN];
    while( 1 ) {
        
        if (read( audio_fd, buf, BUFLEN ) != BUFLEN) {
            printf( "." );
        }
        
        memset( buf, 0x00, BUFLEN );
        if (sendto(s, buf, BUFLEN, 0,
                   (struct sockaddr*) sock_client, socklen) == -1)
            diep("sendto()");

    }

    close(s);
    return 0;
}
