/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/
/*
 * Startup Code for ARM
 *
 * Main File.
 * Contains a very simple main routine and an interrupt handler.
 *
 * by: Clemens Helfmeier
 * ClemensHelfmeier {at] gmx `dot" de
 * (replace the corresponding at and dot in the email, please).
 */

#include "startup_sam7.h"

#include "AT91SAM7S64.h"
#define __inline extern inline
#include "lib_AT91SAM7S64.h"

#include "debug.h"
#define DBG_LEVEL	DBG_LEVEL_VERBOSE
#include "debug.h"

#include "controller/spi.h"
#include "controller/i2s.h"
#include "system/system.h"
#include "system/time.h"
#include "memory/mem.h"
#include "audio/tlv320aic23b.h"

#include "controller/compiler.h"
#include "net/net.h"


/** prototype for function that contains the application
 */
void app_main( void );



/* main()
 * After initialization has been done, this routine will be called.
 * There are no arguments, the routine should never exit!
 */
int main( void ) {
    
	/* enable PIOA */
	AT91F_PIOA_CfgPMC();
	/* everything is input */
	AT91F_PIO_CfgInput( AT91C_BASE_PIOA, 0xFFFFFFFF );

	/* enable external reset line */
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5<<24) | AT91C_SYSC_URSTEN | AT91C_SYSC_BODIEN | ((1<<8) & AT91C_SYSC_ERSTL);
	
	/* debug init .... */
    dbg_init();
    
    DBG_C_S( "\n" );
    {
        char s[20];
        DBG_I_S( "processor speed (calculated at compile time): " ); DBG_I_S( itoa( F_CPU/1000, s, 10 ) ); DBG_I_S( " khz.\n" );
        DBG_I_S( "pll speed (calculated at compile time): " ); DBG_I_S( itoa( F_PLL/1000, s, 10 ) ); DBG_I_S( " khz.\n" );
    }
    

    DBG_I_S( "\nconfiguring hardware modules ... \n" );
    DBG_I_S( "* spi ... " ); spi_init(); DBG_I_S( " done.\n" );
    DBG_I_S( "* i2s ... " ); i2s_init(); DBG_I_S( " done.\n" );
    
    
    DBG_I_S( "\nconfiguring software modules ... \n" );
    DBG_I_S( "* system ... " ); system_init(); DBG_I_S( " done.\n" );
    DBG_I_S( "* time ... " ); time_init(); DBG_I_S( " done.\n" );
    DBG_I_S( "* mem ... " ); mem_init(); DBG_I_S( " done.\n" );
    DBG_I_S( "* net ... " ); if (net_init() != ERROR_SUCCESS) DBG_C_S( "FAILED!\n" ); else DBG_I_S( " done.\n" );
    DBG_I_S( "* audio ... " ); tlv320aic23b_init(); DBG_I_S( " done.\n" );

    
    DBG_I_S( "\nstarting application ... \n" );
    app_main();
    DBG_I_S( "\nApplication exited.\n" );
    
    
    while (1);

}
