/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/



#ifndef _HOSTCONFIG_H_
#define _HOSTCONFIG_H_

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

#define BIG_ENDIAN          1
#define LITTLE_ENDIAN       0

#define ENDIANNESS          LITTLE_ENDIAN

#ifdef ARM
/** ARM OPTIMIZED CODE */
static inline uint32_t byte_swap32(uint32_t v) {
    unsigned int tmp;
    __asm __volatile(
            "eor    %1, %2, %2, ror #16\n"
            "bic    %1, %1, #0x00ff0000\n"
            "mov    %0, %2, ror #8\n"
            "eor    %0, %0, %1, lsr #8"
    : "=r" (v), "=&r" (tmp)
    : "0" (v));
    return (v);
}

static inline uint16_t byte_swap16(uint16_t v) {
    __asm __volatile(
            "mov    %0, %1, ror #8\n"
            "orr    %0, %0, %0, lsr #16\n"
            "bic    %0, %0, %0, lsl #16"
    : "=r" (v)
    : "0" (v));
    return (v);
}
#else
/** NOT OPTIMIZED CODE */
#warning "Not optimized run-time swap functions used."
static inline uint16_t byte_swap16(uint16_t v) {
    return ((((v) & 0xFF00) >> 8) | (((v) & 0xFF)<< 8));
}
static inline uint32_t byte_swap32(uint32_t v) {
    return ((((v) & 0xFF000000) >> 24) | (((v) & 0xFF) << 24) | (((v) & 0xFF0000) >> 8) | (((v) & 0xFF00) << 8));
}
#endif

#ifndef ENDIANNESS
#error "ENDIANNESS not defined!"
#else
#if ENDIANNESS == BIG_ENDIAN
#define HTONS( x )          (x)
#define HTONL( x )          (x)
#define NTOHS( x )          (x)
#define NTOHL( x )          (x)
#define ntohs( x )          (x)
#define ntohl( x )          (x)
#define htons( x )          (x)
#defien htonl( x )          (x)
#elif ENDIANNESS == LITTLE_ENDIAN
#define HTONS( x )          ((((x) & 0xFF00) >> 8) | (((x) & 0xFF)<< 8))
#define HTONL( x )          ((((x) & 0xFF) << 24) | (((x) & 0xFF00) << 8) | (((x) & 0xFF0000)>> 8) | (((x) & 0xFF000000) >> 24))
#define NTOHS( x )          HTONS( x )
#define NTOHL( x )          HTONL( x )
#define htons( x )          byte_swap16( x )
#define htonl( x )          byte_swap32( x )
#define ntohs( x )          byte_swap16( x )
#define ntohl( x )          byte_swap32( x )
#else
#error "ENDIANNESS not defined correctly!"
#endif
#endif
               
#ifdef __cplusplus
}
#endif
#endif
