/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/

/* enc28j60.h
 *
 * this file defines the constants and prototypes for the enc-controller access.
 * it is based on the datasheet and is according to errata rev. 4
 *
 * by:   Clemens Helfmeier
 * date: 13.06.2006
 */

#ifndef _ENC28J60_REGISTER_H_
#define _ENC28J60_REGISTER_H_

/* SPI-Instructions for the controller */
#define INS_RCR		(0x0<<5)
#define INS_RBM		((0x1<<5)|(0x1A))
#define INS_WCR		(0x2<<5)
#define INS_WBM		((0x3<<5)|(0x1A))
#define INS_BFS		(0x4<<5)
#define INS_BFC		(0x5<<5)
#define INS_SWRST	((0x7<<5)|(0x1F))

/* non-Banked register with bit definitions */
#define ECON1		(0x1F)
#define TXRST		(1<<7)
#define RXRST		(1<<6)
#define DMAST		(1<<5)
#define CSUMEN		(1<<4)
#define TXRTS		(1<<3)
#define RXEN		(1<<2)
#define BSEL1		(1<<1)
#define BSEL0		(1<<0)

#define ECON2		(0x1E)
#define AUTOINC		(1<<7)
#define PKTDEC		(1<<6)
#define PWRSV		(1<<5)
#define VRPS		(1<<3)

#define ESTAT		(0x1D)
#define INT			(1<<7)
#define LATECOL		(1<<4)
#define RXBUSY		(1<<2)
#define TXABRT		(1<<1)
#define CLKRDY		(1<<0)

#define EIR			(0x1C)
#define PKTIF		(1<<6)
#define DMAIF		(1<<5)
#define LINKIF		(1<<4)
#define TXIF		(1<<3)
#define WOLIF		(1<<2)
#define TXERIF		(1<<1)
#define RXERIF		(1<<0)

#define EIE			(0x1B)
#define INTIE		(1<<7)
#define PKTIE		(1<<6)
#define DMAIE		(1<<5)
#define LINKIE		(1<<4)
#define TXIE		(1<<3)
#define WOLIE		(1<<2)
#define TXERIE		(1<<1)
#define RXERIE		(1<<0)


/* banked register with bit definitions - BANK 0 */
#define ERDPT		(0x00|(0<<5))
#define EWRPT		(0x02|(0<<5))
#define ETXST		(0x04|(0<<5))
#define ETXND		(0x06|(0<<5))
#define ERXST		(0x08|(0<<5))
#define ERXND		(0x0A|(0<<5))
#define ERXRDPT		(0x0C|(0<<5))
#define ERXWRPT		(0x0E|(0<<5))
#define EDMAST		(0x10|(0<<5))
#define EDMAND		(0x12|(0<<5))
#define EDMADST		(0x14|(0<<5))
#define EDMACS		(0x16|(0<<5))

/* banked register with bit definitions - BANK 1 */
#define EHT0		(0x00|(1<<5))
#define EHT1		(0x01|(1<<5))
#define EHT2		(0x02|(1<<5))
#define EHT3		(0x03|(1<<5))
#define EHT4		(0x04|(1<<5))
#define EHT5		(0x05|(1<<5))
#define EHT6		(0x06|(1<<5))
#define EHT7		(0x07|(1<<5))
#define EPMM0		(0x08|(1<<5))
#define EPMM1		(0x09|(1<<5))
#define EPMM2		(0x0A|(1<<5))
#define EPMM3		(0x0B|(1<<5))
#define EPMM4		(0x0C|(1<<5))
#define EPMM5		(0x0D|(1<<5))
#define EPMM6		(0x0E|(1<<5))
#define EPMM7		(0x0F|(1<<5))
#define EPMCS		(0x10|(1<<5))
#define EPMO		(0x14|(1<<5))

#define EWOLIE		(0x16|(1<<5))
#define UCWOLIE		(1<<7)
#define AWOLIE		(1<<6)
#define PMWOLIE		(1<<4)
#define MPWOLIE		(1<<3)
#define HTWOLIE		(1<<2)
#define MCWOLIE		(1<<1)
#define BCWOLIE		(1<<0)

#define EWOLIR		(0x17|(1<<5))
#define UCWOLIF		(1<<7)
#define AWOLIF		(1<<6)
#define PMWOLIF		(1<<4)
#define MPWOLIF		(1<<3)
#define HTWOLIF		(1<<2)
#define MCWOLIF		(1<<1)
#define BCWOLIF		(1<<0)

#define ERXFCON		(0x18|(1<<5))
#define UCEN		(1<<7)
#define ANDOR		(1<<6)
#define CRCEN		(1<<5)
#define PMEN		(1<<4)
#define MPEN		(1<<3)
#define HTEN		(1<<2)
#define MCEN		(1<<1)
#define BCEN		(1<<0)

#define EPKTCNT		(0x19|(1<<5))


/* banked register with bit definitions - BANK 2 */
#define MACON1		(0x00|(2<<5))
#define LOOPBK		(1<<4)
#define TXPAUS		(1<<3)
#define RXPAUS		(1<<2)
#define PASSALL		(1<<1)
#define MARXEN		(1<<0)

#define MACON2		(0x01|(2<<5))
#define MARST		(1<<7)
#define RNDRST		(1<<6)
#define MARXRST		(1<<3)
#define RFUNRST		(1<<2)
#define MATXRST		(1<<1)
#define TFUNRST		(1<<0)

#define MACON3		(0x02|(2<<5))
#define PADCFG2		(1<<7)
#define PADCFG1		(1<<6)
#define PADCFG0		(1<<5)
#define TXCRCEN		(1<<4)
#define PHDRLEN		(1<<3)
#define HFRMEN		(1<<2)
#define FRMLNEN		(1<<1)
#define FULDPX		(1<<0)

#define MACON4		(0x03|(2<<5))
#define DEFER		(1<<6)
#define BPEN		(1<<5)
#define NOBKOFF		(1<<4)
#define LONGPRE		(1<<1)
#define PUREPRE		(1<<0)


#define MABBIPG		(0x04|(2<<5))
#define MAIPG		(0x06|(2<<5))
#define MACLCON1	(0x08|(2<<5))
#define MACLCON2	(0x09|(2<<5))
#define MAMXFL		(0x0A|(2<<5))

#define MAPHSUP		(0x0D|(2<<5))
#define RSTINTFC	(1<<7)
#define RSTRMII		(1<<3)

#define MICON		(0x11|(2<<5))
#define RSTMII		(1<<7)

#define MICMD		(0x12|(2<<5))
#define MIISCAN		(1<<1)
#define MIIRD		(1<<0)

#define MIREGADR	(0x14|(2<<5))
#define MIWR		(0x16|(2<<5))
#define MIRD		(0x18|(2<<5))

/* banked register with bit definitions - BANK 3 */
#define MAADR0		(0x04|(3<<5))
#define MAADR1		(0x05|(3<<5))
#define MAADR2		(0x02|(3<<5))
#define MAADR3		(0x03|(3<<5))
#define MAADR4		(0x00|(3<<5))
#define MAADR5		(0x01|(3<<5))
#define EBSTSD		(0x06|(3<<5))

#define EBSTCON		(0x07|(3<<5))
#define PSV2		(1<<7)
#define PSV1		(1<<6)
#define PSV0		(1<<5)
#define PSEL		(1<<4)
#define TMSEL1		(1<<3)
#define TMSEL0		(1<<2)
#define TME			(1<<1)
#define BISTST		(1<<0)

#define EBSTCS		(0x08|(3<<5))

#define MISTAT		(0x0A|(3<<5))
#define NVALID		(1<<2)
#define SCAN		(1<<1)
#define BUSY		(1<<0)

#define EREVID		(0x12|(3<<5))

#define ECOCON		(0x15|(3<<5))
#define COCON2		(1<<2)
#define COCON1		(1<<1)
#define COCON0		(1<<0)

#define EFLOCON		(0x17|(3<<5))
#define FULDPXS		(1<<2)
#define FCEN1		(1<<1)
#define FCEN0		(1<<0)

#define EPAUS		(0x18|(3<<5))

/* phy registers */
#define PHCON1		0x00
#define PRST		(1<<15)
#define PLOOPBK		(1<<14)
#define PPWRSV		(1<<11)
#define PDPXMD		(1<<8)

#define PHSTAT1		0x01
#define PFDPX		(1<<12)
#define PHDPX		(1<<11)
#define LLSTAT		(1<<2)
#define JBSTAT		(1<<1)

#define PHID1		0x02
#define PHID2		0x03

#define PHCON2		0x10
#define HDLDIS		(1<<8)
#define JABBER		(1<<10)
#define TXDIS		(1<<13)
#define FRCLNK		(1<<14)

#define PHSTAT2		0x11
#define PHIE		0x12
#define PHIR		0x13
#define PHLCON		0x14
#define LEDB_DISPLAY_RECIEVE_ACTIVITY	(0x02<<4)
#define LEDA_DISPLAY_LINK_STATUS	(0x04<<8)
#define STRETCH_139	(0x02<<2)
#define STRCH		(0x01<<0)

/* TX Control Byte definitions */
#define PHUGEEN		(1<<3)
#define PPADEN		(1<<2)
#define PCRCEN		(1<<1)
#define POVERRIDE	(1<<0)

#include "controller/spi.h"
#include "controller/config_spi.h"
#define enc28j60_dev    SPI_1


#ifdef __cplusplus
extern "C" {
#endif


/* Prototypes */
/* ETHERNET REGISTER MODIFYING FUNCTIONS */
unsigned char read_eth_reg( unsigned char addr );
static inline unsigned short read_eth_reg16( unsigned char addr ) {
    return read_eth_reg( addr ) | (read_eth_reg(addr+1)<<8);
}
void write_eth_reg( unsigned char addr, unsigned char data );
static inline void write_eth_reg16( unsigned char addr, unsigned short data ) {
    /* write lower byte first, always. */
    write_eth_reg( addr, data & 0xFF );
    write_eth_reg( addr+1, data>>8 );
}

/* MII REGISTER */
unsigned char read_mii_reg( unsigned char addr );
static inline unsigned short read_mii_reg16( unsigned char addr ) {
    return read_mii_reg( addr ) | (read_mii_reg(addr+1)<<8);
}
static inline void write_mii_reg( unsigned char addr, unsigned char data ) {
    write_eth_reg( addr, data );
}
static inline void write_mii_reg16( unsigned char addr, unsigned short data ) {
    write_eth_reg16( addr, data );
}

/* MAC REGISTER */
static inline unsigned char read_mac_reg( unsigned char addr ) {
    return read_mii_reg( addr );
}
static inline unsigned short read_mac_reg16( unsigned char addr ) {
    return read_mii_reg16( addr );
}
static inline void write_mac_reg( unsigned char addr, unsigned char data ) {
    write_eth_reg( addr, data );
}
static inline void write_mac_reg16( unsigned char addr, unsigned short data ) {
    write_eth_reg16( addr, data );
}

/* PHY REGISTER */
unsigned short read_phy_reg16( unsigned char addr );
void write_phy_reg16( unsigned char addr, unsigned short data );

void write_mem( void *data, unsigned short addr, unsigned short len );
void read_mem( void *data, unsigned short addr, unsigned short len );

unsigned short crc16( unsigned short start, unsigned short end );

void clear_bits( unsigned char addr, unsigned char bits );
void set_bits( unsigned char addr, unsigned char bits );

void soft_reset( void );
void set_bank( unsigned char bank);

unsigned int link_status( void );

#ifdef __cplusplus
}
#endif
#endif
