/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/



#include "dns.h"
#include "udp.h"

#include "memory/mem.h"

#include <string.h>


typedef struct __attribute__ ((__packed__) ){
    uint16_t    id;
    uint16_t    flags;
    uint16_t    qdcount;
    uint16_t    ancount;
    uint16_t    nscount;
    uint16_t    arcount;
} dns_header;


/** strip anything from the name that is not part of the real domain name.
 * this is any path appedned (after a slash) and any protocol before the domain name
 */
ERROR dns_domain_part( char * name, char ** domainname, unsigned int * domainlen ) {
    char * s = name;
    // by default the domain begins at the beginning of the string
    *domainname = s;
    while (*s != 0) {
        if (*s == ':') {
            // here is a colon, strip the following slashes if there are any:
            while (*(++s) == '/');
            *domainname = s;    // ok, the beginning of domain name is here
        }
        if (*s == '/') {
            // any slash that is not after a colon terminates the domain name.
            return ERROR_SUCCESS;
        }
        if (!((*s == '.') || ((*s >= '0') && (*s <= '9')) || ((*s >= 'a') && (*s <= 'z')) || ((*s >='A') && (*s <='Z')))) {
            // this is neither a digit nor a character or a dot -> invalid character in domain name
            return ERROR_DNS_INVALID_NAME;
        }
        *domainlen++;
        s++;
    }
    // no trailing slash was encountered -> domain name valid.
    return ERROR_SUCCESS;
}


/** send a query to the name server
 */
ERROR dns_query( char * domain ) {
    char * sname = domain;
    unsigned int slen = 0;
    ERROR ret;
    // check if domain is valid
    ret = dns_domain_part( domain, &sname, &slen );
    if (ret != ERROR_SUCCESS) return ret;
    // ok, compose the query
    
    
}
