/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/


#include "system/error.h"
#include "memory/mem.h"
#include <string.h>

#include "net/netconfig.h"
#include "net/proto/icmp.h"

#include "net/netutils.h"

        
#include "debug/debug.h"
#define DBG_LEVEL   DBG_LEVEL_ERROR
#include "debug/debug.h"

        
        
/// ICMP TYPE FIELD
#define ICMP_REQUEST    0x08
#define ICMP_REPLY      0x00
#define ICMP_CODE       0x00
#define ICMP_DSTUNR     0x03


ERROR icmp_init( void ) {
    
    return ERROR_SUCCESS;
}

/** handle an incoming icmp message
 */
void icmp_input_handler( void * mem, unsigned int len ) {
    icmp_header * phdr = (icmp_header*)(mem + ICMP_OFFSET - ICMP_HEADER_SIZE);
    if (len < ICMP_TOTAL_SIZE) return;  // discard any packet that is too short
    // what to do with this packet?
    if ((phdr->type == ICMP_REQUEST) && (phdr->code == ICMP_CODE)) {
        // answer this packet
        // keep the packet and modify it for the answer:
//        mem_use( mem );
        phdr->type = ICMP_REPLY;
        phdr->crc = 0;
        phdr->crc = htons(ICMP_CRC( phdr, len - ICMP_TOTAL_SIZE + ICMP_HEADER_SIZE )); // calculate crc over header and payload
        ipv4_address src = ICMP_SRC( mem );
        ERROR ret = ICMP_SEND_PACKET( mem, len, src );
        if (ret != ERROR_SUCCESS) {
            DBG_E_S( "icmp_input_handler: failed to send reply: " ); DBG_E_I( ret ); DBG_E_S( ".\n" );
        } else {
            DBG_V_S( "icmp_input_handler: ping request answered.\n" );
        }
//        mem_free( mem );
    }
}

/** icmp_dstunr send an error message back to the source of the received packet. 
 * mem, len is the original packet. The original packet MUST be an IPV4 packet!
 */
ERROR icmp_dstunr( void * mem, unsigned int len, uint8_t code ) {
    if (IPV4_TOTAL_SIZE > len ) return ERROR_FAILURE;
    void * rmem = mem_allocate( ICMP_TOTAL_SIZE /*packet header*/ + IPV4_HEADER_SIZE /*ipv4 header of original packet*/ + 8 /* 64bits of payload of original packet*/ );
    unsigned int cpl;
    if (IPV4_TOTAL_SIZE + 8 > len) {
        cpl = IPV4_HEADER_SIZE + (len - IPV4_TOTAL_SIZE); /* ipv4 payload length */
    } else {
        cpl = IPV4_HEADER_SIZE + 8;
    }
    // fill icmp payload
    memcpy( rmem + ICMP_OFFSET, mem + IPV4_OFFSET - IPV4_HEADER_SIZE, cpl );
    // fill icmp header
    icmp_header * hdr = rmem + ICMP_OFFSET - ICMP_HEADER_SIZE;
    hdr->type = ICMP_DSTUNR;
    hdr->code = code;
    hdr->crc = 0;
    hdr->crc = htons(ICMP_CRC( hdr, ICMP_HEADER_SIZE + cpl ));
    ERROR ret = ICMP_SEND_PACKET( rmem, ICMP_TOTAL_SIZE + cpl, IPV4_SRC( mem ) );
    mem_free( rmem );
    return ret;
}
    
    


