/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/



#ifndef _UDP_H_
#define _UDP_H_

#include <stdint.h>

#include "net/net/ipv4.h"

#ifdef __cplusplus
extern "C" {
#endif

/// should the checksum be added to the udp header?
#define UDP_DOCHECKSUM      0
    
/// maximum number of sockets supported
#define UDP_SOCKET_COUNT    8
/// number of packets that can be queued on one udp socket
#define UDP_QUEUE_LEN       64
    
    
typedef uint16_t udp_port;
typedef ipv4_address udp_address;


typedef struct {
    udp_address remote;            ///< remote ip address
    udp_address msk;            ///< remote ip address mask (for listening sockets)
    udp_port localport;         ///< local port
    udp_port remoteport;        ///< remote port
    unsigned int rindex;        ///< queue index for reading
    unsigned int qlen;          ///< length of queue
    struct {
        void * mem;
        unsigned int len;
    } buffer[UDP_QUEUE_LEN];    ///< buffer queue for holding recieved packets
} udp_socket;

typedef struct __attribute__ ((__packed__)){
    udp_port src_port;          ///< source port
    udp_port dst_port;          ///< destination port
    uint16_t len;               ///< length of header and packet
    uint16_t crc;               ///< crc of packet and header
} udp_header;



ERROR udp_init( void );


void udp_input_handler( void * mem, unsigned int len );

/** open a new socket for listening */
ERROR udp_listen( udp_socket * s, udp_address src, udp_address msk, udp_port localport, udp_port remoteport );
/** read from a socket */
ERROR udp_recv( udp_socket * s, void ** mem, unsigned int * len );
/** write to a socket */
ERROR udp_send( udp_socket * s, void * mem, unsigned int len );

/** number of packets in fifo */
static inline unsigned int udp_fifo_fill( udp_socket * s ) {
    return s->qlen;
}


#ifdef __cplusplus
}
#endif
#endif
