/*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Copyright (C) 2006 by Clemens Helfmeier,
    Clemens(at)Helfmeier[dot]de

*/

#include "utils/reorder.h"
#include "system/error.h"
#include "memory/mem.h"

struct _order_entry {
    struct _order_entry * next;
    void * mem;
    unsigned int len;
    unsigned int index;     // this is the order criteria.
};
typedef struct _order_entry order_entry;
    
    

order_entry * order_first;
unsigned int order_count;

ERROR reorder_init( void ) {
    order_first = 0;
    order_count = 0;
}

unsigned int reorder_count( void ) {
    return order_count;
}

ERROR reorder_put( void * mem, unsigned int len, unsigned int i ) {
    // create a new order entry
    order_entry * no = mem_allocate( sizeof(order_entry) );
    if (no == 0) return ERROR_OUT_OF_MEMORY;
    // fill the memory into the order entry
    no->mem = mem;
    no->len = len;
    no->index = i;
    // check where to insert the new order entry
    if (order_first == 0) {
        // this is the first entry we insert.
        order_first = no;
        no->next = 0;
        order_count++;
        return ERROR_SUCCESS;
    }
    // walk through the chain and insert the entry
    order_entry * o = order_first, * lo = 0;
    while (o != 0) {
        if (o->index == i) {
            return ERROR_FAILURE;   // this is already there...
        }
        if (o->index - order_first->index > i - order_first->index) {
            // o should be sorted behind i
            no->next = lo->next;
            lo->next = no;
            order_count++;
            return ERROR_SUCCESS;
        }
        lo = o;
        o = o->next;
    }
    // add the entry to the end:
    no->next = 0;
    lo->next = no;
    order_count++;
    return ERROR_SUCCESS;
}

ERROR reorder_read( void ** mem, unsigned int * len ) {
    // return the first entry
    if (order_first != 0) {
        *mem = order_first->mem;
        *len = order_first->len;
        order_first = order_first->next;
        order_count--;
        return ERROR_SUCCESS;
    }
    return ERROR_FAILURE;
}

